MatrixTileSlice

Struct MatrixTileSlice 

pub struct MatrixTileSlice<'a> {
    limbs: *const u64,
    dimensions: [usize; 2],
    stride: NonZeroUsize,
    _marker: PhantomData<&'a ()>,
}
An immutable view of a tile within a matrix.

A tile is a rectangular region composed of multiple 64 x 64 blocks. Tiles enable hierarchical parallelization: large matrices are divided into tiles which are processed in parallel, and each tile is further divided into blocks for vectorization.

Safety

The limbs pointer must remain valid for the lifetime 'a, and must point to a region large enough for dimensions[0] * 64 rows and dimensions[1] blocks with the given stride.

Fields

limbs: *const u64dimensions: [usize; 2]

Dimensions of the tile in units of 64 x 64 blocks: [block_rows, block_cols]

stride: NonZeroUsize

Number of limbs between consecutive rows in the parent matrix

_marker: PhantomData<&'a ()>

Implementations

Source

impl<'a> MatrixTileSlice<'a>

pub fn from_matrix(m: &'a Matrix) -> Self

pub fn block_rows(&self) -> usize

Returns the number of 64 x 64 block rows in this tile.

pub fn block_columns(&self) -> usize

Returns the number of 64 x 64 block columns in this tile.

pub fn block_at( &self, block_row: usize, block_col: usize, ) -> MatrixBlockSlice<'_>

Returns a view of the block at the given block coordinates.

Panics

Panics if the block coordinates are out of bounds.

pub fn split_rows_at( &self, block_rows: usize, ) -> (MatrixTileSlice<'_>, MatrixTileSlice<'_>)

pub fn split_columns_at( &self, block_columns: usize, ) -> (MatrixTileSlice<'_>, MatrixTileSlice<'_>)