FieldElement

Struct FieldElement 

pub struct FieldElement<F: FieldInternal> {
    field: F,
    pub(super) value: F::ElementContainer,
}
An element of a field.

This contains the field itself so that it knows how to do arithmetic operations. We want this to be a struct rather than a trait, which means that we want the actual storage of the value to be managed by the field itself. Therefore, we have an internal field trait that knows about arithmetic operations and other implementation details, but these operations are only accessible from outside the crate using this struct.

It might seem wasteful to handle, say, FieldElement<Fp<P>>s rather than u32 in the API for FqVector<Fp<P>>. However, this gives us type-level guarantees that the invariants of the elements hold, i.e. in this case that its value is in the range 0..P. Moreover, this is bigger than a bare F::ElementContainer only when the field has a positive memory footprint. The cases we care most about, Fp<P2>, Fp<P3>, Fp<P5>, and Fp<P7>, are all ZSTs and therefore don’t cause any overhead.

Fields

field: Fvalue: F::ElementContainer

Implementations

Source

impl<F: FieldInternal> FieldElement<F>

pub(super) fn new(field: F, value: F::ElementContainer) -> Self

Create a new field element. This is only visible to the field module, because the caller is responsible for ensuring that the invariants of value hold.

Handling FieldElements in the API rather than the containers directly has the advantage of being sure at compile-time that the invariants hold.

pub fn field(&self) -> F

pub(crate) fn val(self) -> F::ElementContainer

pub fn inv(self) -> Option<Self>

pub fn frobenius(self) -> Self