FieldElementContainer

Trait FieldElementContainer 

pub trait FieldElementContainer:
    Debug
    + Display
    + Clone
    + PartialEq
    + Eq
    + Hash { }
This just ensures that the containers are “nice enough”, in the sense that they are cloneable, hashable, etc. We may add more custom methods in the future.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementations on Foreign Types

Source

impl FieldElementContainer for u32

Implementors