fn zech_logs<P: Prime>(p: P, d: u32) -> &'static [SmallFqElement]Expand description
Return the Zech logarithm table for the given field. If it does not exist yet, initialize it. The initialization might be fairly expensive (several ms).
fn zech_logs<P: Prime>(p: P, d: u32) -> &'static [SmallFqElement]Return the Zech logarithm table for the given field. If it does not exist yet, initialize it. The initialization might be fairly expensive (several ms).