SmallFq

Struct SmallFq 

pub struct SmallFq<P> {
    p: P,
    d: u32,
    q: u32,
    table: &'static [SmallFqElement],
}
A field of order q = p^d, where q < 2^16 and d > 1. Fields of that size are small enough that we can cache their Zech logarithms.

Note: This populates the Zech logarithm table eagerly, which can be rather expensive (several milliseconds). Only construct these fields if you’re going to use them.

Fields

p: Pd: u32q: u32table: &'static [SmallFqElement]

Implementations

Source

impl<P: Prime> SmallFq<P>

pub fn new(p: P, d: u32) -> Self

pub fn negative_one(self) -> FieldElement<Self>

Return the element -1. If p = 2, this is a^0 = 1. Otherwise, it is a^((q - 1) / 2).

pub fn a(self) -> FieldElement<Self>

The distinguished primitive element that generates the multiplicative group of the field.