FpElement

Struct FpElement 

pub struct FpElement<MOD, const LIMBS: usize>
where MOD: ConstPrimeMontyParams<LIMBS>,
{ /* private fields */ }
An element of the prime field $\mathbb{F}_p = \mathbb{Z}/p\mathbb{Z}$, stored in Montgomery form.

The internal value uses crypto-bigint’s ConstMontyForm, so arithmetic is performed in Montgomery representation while the public constructors and accessors accept and return canonical integers.

Implementations

Source

impl<MOD, const LIMBS: usize> FpElement<MOD, LIMBS>
where MOD: ConstPrimeMontyParams<LIMBS>,

pub fn from_uint(x: Uint<LIMBS>) -> Self

Goes from an Uint<LIMBS> to an element of $\mathbb{F}_p$

Arguments
  • x - An integer (type: Uint<LIMBS>)
Returns

Element of $\mathbb{F}_p$ (type: Self)

pub fn from_words(words: [u64; LIMBS]) -> Self

Goes from an [u64; LIMBS] to an element of $\mathbb{F}_p$

Arguments
  • words - A vec of u64 (type: [u64; LIMBS])
Returns

Element of $\mathbb{F}_p$ (type: Self)

pub fn from_limbs(limbs: &[u64]) -> Self

Goes from an &[u64] to an element of $\mathbb{F}_p$

Arguments
  • limbs - A vec of u64 (type: &[u64])
Returns

Element of $\mathbb{F}_p$ (type: Self)

pub fn from_u64(val: u64) -> Self

Goes from an u64 to an element of $\mathbb{F}_p$

Arguments
  • val - A u64 int (type: u64)
Returns

Element of $\mathbb{F}_p$ (type: Self)

pub fn as_uint(&self) -> Uint<LIMBS>

Goes from an element of $\mathbb{F}_p$ to the corresponding Uint

Arguments
  • &self - Element of $\mathbb{F}_p$ (type: Self)
Returns

The corresponding Uint (type: Uint<LIMBS>)

pub fn as_limbs(&self) -> [u64; LIMBS]

Goes from an element of $\mathbb{F}_p$ to the limbs

Arguments
  • &self - Element of $\mathbb{F}_p$ (type: Self)
Returns

The limbs giving the integer (type: [u64; LIMBS])

pub fn to_montgomery(&self) -> Uint<LIMBS>

Gives a montgomery Uint<LIMBS> from an $\mathbb{F}_p$ element

Arguments
  • &self - Element of $\mathbb{F}_p$ (type: Self)
Returns

Integer in montgomery from (type: Uint<LIMBS>)

pub fn from_montgomery(mont: Uint<LIMBS>) -> Self

Gives an element of $\mathbb{F}_p$ from the Montgomery representation.

Arguments
  • mont - The input montgomery value (type: Uint<LIMBS>)
Returns

Corresponding element of $\mathbb{F}_p$ (type: Self)