dispatch_struct

Macro dispatch_struct 

macro_rules! dispatch_struct {
    ($(#[derive $derives:tt])? $vis:vis $name:ident $(<$life:lifetime>)? from $fq_name:ident) => { ... };
}
Define a struct that wraps some field-dependent type in an enum. Since Fp<P2> through Fp<P7> are all ZSTs, working with them under the hood is in theory much faster than working with an arbitrary prime field, let alone finite field.