FqSliceMut

Struct FqSliceMut 

pub struct FqSliceMut<'a, F: Field> {
    fq: F,
    limbs: &'a mut [u64],
    start: usize,
    end: usize,
}
A mutable slice of an FqVector.

This mutably borrows the vector. Since it is a mutable borrow, it cannot implement Copy. However, it has a FqSliceMut::copy function that imitates the reborrowing, that mutably borrows FqSliceMut and returns a FqSliceMut with a shorter lifetime.

Fields

fq: Flimbs: &'a mut [u64]start: usizeend: usize

Implementations

Source

impl<'a, F: Field> FqSliceMut<'a, F>

pub(super) fn new(fq: F, limbs: &'a mut [u64], start: usize, end: usize) -> Self

pub fn fq(&self) -> F

pub(super) fn start(&self) -> usize

pub(super) fn end(&self) -> usize

pub(super) fn end_mut(&mut self) -> &mut usize

pub(super) fn limbs(&self) -> &[u64]

pub(super) fn limbs_mut(&mut self) -> &mut [u64]

Source

impl<F: Field> FqSliceMut<'_, F>

pub(super) fn scale_helper(&mut self, c: F::ElementContainer)

pub(super) fn add_helper( &mut self, other: FqSlice<'_, F>, c: F::ElementContainer, )

pub(super) fn add_offset_helper( &mut self, other: FqSlice<'_, F>, c: F::ElementContainer, offset: usize, )

pub(super) fn set_entry_helper( &mut self, index: usize, value: F::ElementContainer, )

pub(super) fn add_basis_element_helper( &mut self, index: usize, value: F::ElementContainer, )

pub(super) fn add_masked_helper( &mut self, other: FqSlice<'_, F>, c: F::ElementContainer, mask: &[usize], )

pub(super) fn add_unmasked_helper( &mut self, other: FqSlice<'_, F>, c: F::ElementContainer, mask: &[usize], )

pub(super) fn add_tensor_helper( &mut self, offset: usize, coeff: F::ElementContainer, left: FqSlice<'_, F>, right: FqSlice<'_, F>, )

Source

impl<'a, F: Field> FqSliceMut<'a, F>

pub fn prime(&self) -> ValidPrime

pub fn add_basis_element(&mut self, index: usize, value: FieldElement<F>)

pub fn set_entry(&mut self, index: usize, value: FieldElement<F>)

fn reduce_limbs(&mut self)

pub fn scale(&mut self, c: FieldElement<F>)

pub fn set_to_zero(&mut self)

pub fn add(&mut self, other: FqSlice<'_, F>, c: FieldElement<F>)

pub fn add_offset( &mut self, other: FqSlice<'_, F>, c: FieldElement<F>, offset: usize, )

pub fn add_tensor( &mut self, offset: usize, coeff: FieldElement<F>, left: FqSlice<'_, F>, right: FqSlice<'_, F>, )

Adds v otimes w to self.

pub fn assign(&mut self, other: FqSlice<'_, F>)

TODO: improve efficiency

pub fn shl_assign(&mut self, shift: usize)

Shifts the entries of self to the left by shift entries.

pub fn add_shift_none(&mut self, other: FqSlice<'_, F>, c: FieldElement<F>)

Adds c * other to self. other must have the same length, offset, and prime as self.

fn add_shift_left(&mut self, other: FqSlice<'_, F>, c: FieldElement<F>)

fn add_shift_right(&mut self, other: FqSlice<'_, F>, c: FieldElement<F>)

pub fn add_masked( &mut self, other: FqSlice<'_, F>, c: FieldElement<F>, mask: &[usize], )

Given a mask v, add the v[i]th entry of other to the ith entry of self.

pub fn add_unmasked( &mut self, other: FqSlice<'_, F>, c: FieldElement<F>, mask: &[usize], )

Given a mask v, add the ith entry of other to the v[i]th entry of self.

pub fn slice_mut(&mut self, start: usize, end: usize) -> FqSliceMut<'_, F>

pub fn as_slice(&self) -> FqSlice<'_, F>

pub fn copy(&mut self) -> FqSliceMut<'_, F>

Generates a version of itself with a shorter lifetime