pub struct MatrixTileSliceMut<'a> {
limbs: *mut u64,
dimensions: [usize; 2],
stride: NonZeroUsize,
_marker: PhantomData<&'a ()>,
}Expand description
A mutable view of a tile within a matrix.
Safety
The limbs pointer must remain valid and exclusively accessible for the lifetime 'a, and must
point to a region large enough for dimensions[0] * 64 rows and dimensions[1] blocks with the
given stride.
Fields
limbs: *mut u64dimensions: [usize; 2]Dimensions of the tile in units of 64 x 64 blocks: [block_rows, block_cols]
stride: NonZeroUsizeNumber of limbs between consecutive rows in the parent matrix
_marker: PhantomData<&'a ()>Implementations
Sourceimpl<'a> MatrixTileSliceMut<'a>
impl<'a> MatrixTileSliceMut<'a>
pub fn from_matrix(m: &'a mut Matrix) -> Self
pub fn block_rows(&self) -> usize
pub fn block_columns(&self) -> usize
pub fn block_mut_at( &mut self, block_row: usize, block_col: usize, ) -> MatrixBlockSliceMut<'_>
pub fn split_rows_at_mut( &mut self, block_rows: usize, ) -> (MatrixTileSliceMut<'_>, MatrixTileSliceMut<'_>)
pub fn split_columns_at_mut( &mut self, block_columns: usize, ) -> (MatrixTileSliceMut<'_>, MatrixTileSliceMut<'_>)
pub fn zero_out(&mut self)