pub trait FieldElementContainer:
Debug
+ Display
+ Clone
+ PartialEq
+ Eq
+ Hash { }Expand description
This just ensures that the containers are “nice enough”, in the sense that they are cloneable, hashable, etc. We may add more custom methods in the future.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.