pub struct SmallFq<P> {
p: P,
d: u32,
q: u32,
table: &'static [SmallFqElement],
}Expand description
A field of order q = p^d, where q < 2^16 and d > 1. Fields of that size are small enough
that we can cache their Zech logarithms.
Note: This populates the Zech logarithm table eagerly, which can be rather expensive (several milliseconds). Only construct these fields if you’re going to use them.
Fields
p: Pd: u32q: u32table: &'static [SmallFqElement]Implementations
Sourceimpl<P: Prime> SmallFq<P>
impl<P: Prime> SmallFq<P>
pub fn new(p: P, d: u32) -> Self
Sourcepub fn negative_one(self) -> FieldElement<Self>
pub fn negative_one(self) -> FieldElement<Self>
Return the element -1. If p = 2, this is a^0 = 1. Otherwise, it is a^((q - 1) / 2).
Sourcepub fn a(self) -> FieldElement<Self>
pub fn a(self) -> FieldElement<Self>
The distinguished primitive element that generates the multiplicative group of the field.